@@ -32,8 +32,7 @@
   (autoload 'message-make-message-id "message"))
 
 (defvar mml-generate-multipart-alist
-  '(("signed" . rfc2015-generate-signed-multipart)
-    ("encrypted" . rfc2015-generate-encrypted-multipart))
+  nil
   "*Alist of multipart generation functions.
 
 Each entry has the form (NAME . FUNCTION), where
@@ -64,6 +63,13 @@ suggestion each time.  The function is called with one parameter,
 which is a number that says how many times the function has been
 called for this message.")
 
+(defvar mml-confirmation-set nil
+  "A list of symbols, each of which disables some warning.
+`unknown-encoding': always send messages contain characters with
+unknown encoding; `use-ascii': always use ASCII for those characters
+with unknown encoding; `multipart': always send messages with more than
+one charsets.")
+
 (defun mml-parse ()
   "Parse the current buffer as an MML document."
   (goto-char (point-min))
@@ -76,7 +82,7 @@ called for this message.")
 
 (defun mml-parse-1 ()
   "Parse the current buffer as an MML document."
-  (let (struct tag point contents charsets warn)
+  (let (struct tag point contents charsets warn use-ascii)
     (while (and (not (eobp))
 		(not (looking-at "<#/multipart")))
       (cond
@@ -93,12 +99,23 @@ called for this message.")
 	(setq point (point)
 	      contents (mml-read-part)
 	      charsets (mm-find-mime-charset-region point (point)))
+	(when (memq nil charsets)
+	  (if (or (memq 'unknown-encoding mml-confirmation-set)
+		  (y-or-n-p
+		   "Warning: You message contains characters with unknown encoding. Really send?"))
+	      (if (setq use-ascii 
+			(or (memq 'use-ascii mml-confirmation-set)
+			    (y-or-n-p "Use ASCII as charset?")))
+		  (setq charsets (delq nil charsets))
+		(setq warn nil))
+	    (error "Edit your message to remove those characters")))
 	(if (< (length charsets) 2)
 	    (push (nconc tag (list (cons 'contents contents)))
 		  struct)
 	  (let ((nstruct (mml-parse-singlepart-with-multiple-charsets
-			  tag point (point))))
+			  tag point (point) use-ascii)))
 	    (when (and warn
+		       (not (memq 'multipart mml-confirmation-set))
 		       (not
 			(y-or-n-p
 			 (format
@@ -110,17 +127,20 @@ called for this message.")
       (forward-line 1))
     (nreverse struct)))
 
-(defun mml-parse-singlepart-with-multiple-charsets (orig-tag beg end)
+(defun mml-parse-singlepart-with-multiple-charsets 
+  (orig-tag beg end &optional use-ascii)
   (save-excursion
     (narrow-to-region beg end)
     (goto-char (point-min))
-    (let ((current (mm-mime-charset (char-charset (following-char))))
+    (let ((current (or (mm-mime-charset (mm-charset-after))
+		       (and use-ascii 'us-ascii)))
 	  charset struct space newline paragraph)
       (while (not (eobp))
 	(cond
 	 ;; The charset remains the same.
-	 ((or (eq (setq charset (mm-mime-charset
-				 (char-charset (following-char)))) 'us-ascii)
+	 ((or (eq (setq charset (mm-mime-charset (mm-charset-after))) 
+		  'us-ascii)
+	      (and use-ascii (not charset))
 	      (eq charset current)))
 	 ;; The initial charset was ascii.
 	 ((eq current 'us-ascii)
@@ -241,7 +261,8 @@ called for this message.")
 		  (delete-region (+ (match-beginning 0) 2)
 				 (+ (match-beginning 0) 3))))))
 	    (setq charset (mm-encode-body))
-	    (setq encoding (mm-body-encoding charset))
+	    (setq encoding (mm-body-encoding charset 
+					     (cdr (assq 'encoding cont))))
 	    (setq coded (buffer-string)))
 	(mm-with-unibyte-buffer
 	  (cond
@@ -300,7 +321,6 @@ called for this message.")
         (let ((mml-boundary (mml-compute-boundary cont)))
           (insert (format "Content-Type: multipart/%s; boundary=\"%s\"\n"
                           type mml-boundary))
-          (insert "\n")
           (setq cont (cddr cont))
           (while cont
             (insert "\n--" mml-boundary "\n")
@@ -596,7 +616,7 @@ called for this message.")
 		  (format "Content type (default %s): " default)
 		  (mapcar
 		   'list
-		   (delete-duplicates
+		   (mm-delete-duplicates
 		    (nconc
 		     (mapcar (lambda (m) (cdr m))
 			     mailcap-mime-extensions)
@@ -613,8 +633,7 @@ called for this message.")
 					nil
 				      type)))
 				(cdr l))))
-		       mailcap-mime-data)))
-		    :test 'equal)))))
+		       mailcap-mime-data))))))))
     (if (not (equal string ""))
 	string
       default)))
@@ -654,7 +673,14 @@ called for this message.")
 	(when (string-match "[\"\\~/* \t\n]" value)
 	  (setq value (prin1-to-string value)))
 	(insert (format " %s=%s" key value)))))
-  (insert ">\n<#/" name ">\n"))
+  (insert ">\n"))
+
+(defun mml-insert-empty-tag (name &rest plist)
+  "Insert an empty MML tag described by NAME and PLIST."
+  (when (symbolp name)
+    (setq name (symbol-name name)))
+  (apply #'mml-insert-tag name plist)
+  (insert "<#/" name ">\n"))
 
 ;;; Attachment functions.
 
@@ -671,8 +697,8 @@ description of the attachment."
 	  (type (mml-minibuffer-read-type file))
 	  (description (mml-minibuffer-read-description)))
      (list file type description)))
-  (mml-insert-tag 'part 'type type 'filename file 'disposition "attachment"
-		  'description description))
+  (mml-insert-empty-tag 'part 'type type 'filename file
+			'disposition "attachment" 'description description))
 
 (defun mml-attach-buffer (buffer &optional type description)
   "Attach a buffer to the outgoing MIME message.
@@ -682,8 +708,8 @@ See `mml-attach-file' for details of operation."
 	  (type (mml-minibuffer-read-type buffer "text/plain"))
 	  (description (mml-minibuffer-read-description)))
      (list buffer type description)))
-  (mml-insert-tag 'part 'type type 'buffer buffer 'disposition "attachment"
-		  'description description))
+  (mml-insert-empty-tag 'part 'type type 'buffer buffer
+			'disposition "attachment" 'description description))
 
 (defun mml-attach-external (file &optional type description)
   "Attach an external file into the buffer.
@@ -694,40 +720,46 @@ TYPE is the MIME type to use."
 	  (type (mml-minibuffer-read-type file))
 	  (description (mml-minibuffer-read-description)))
      (list file type description)))
-  (mml-insert-tag 'external 'type type 'name file 'disposition "attachment"
-		  'description description))
+  (mml-insert-empty-tag 'external 'type type 'name file
+			'disposition "attachment" 'description description))
 
 (defun mml-insert-multipart (&optional type)
   (interactive (list (completing-read "Multipart type (default mixed): "
-		     '(("mixed") ("alternative") ("digest") ("parallel")
-		       ("signed") ("encrypted"))
-		     nil nil "mixed")))
+				      '(("mixed") ("alternative") ("digest") ("parallel")
+					("signed") ("encrypted"))
+				      nil nil "mixed")))
   (or type
       (setq type "mixed"))
-  (mml-insert-tag "multipart" 'type type)
+  (mml-insert-empty-tag "multipart" 'type type)
+  (forward-line -1))
+
+(defun mml-insert-part (&optional type)
+  (interactive
+   (list (mml-minibuffer-read-type "")))
+  (mml-insert-tag 'part 'type type 'disposition "inline")
   (forward-line -1))
 
 (defun mml-preview (&optional raw)
- "Display current buffer with Gnus, in a new buffer.
+  "Display current buffer with Gnus, in a new buffer.
 If RAW, don't highlight the article."
- (interactive "P")
- (let ((buf (current-buffer)))
-   (switch-to-buffer (get-buffer-create 
-                     (concat (if raw "*Raw MIME preview of "
-                               "*MIME preview of ") (buffer-name))))
-   (erase-buffer)
-   (insert-buffer buf)
-   (if (re-search-forward
-	(concat "^" (regexp-quote mail-header-separator) "\n") nil t)
-       (replace-match "\n"))
-   (mml-to-mime)
-   (unless raw
-     (run-hooks 'gnus-article-decode-hook)
-     (let ((gnus-newsgroup-name "dummy"))
-      (gnus-article-prepare-display)))
-   (fundamental-mode)
-   (setq buffer-read-only t)
-   (goto-char (point-min))))
+  (interactive "P")
+  (let ((buf (current-buffer)))
+    (switch-to-buffer (get-buffer-create 
+		       (concat (if raw "*Raw MIME preview of "
+				 "*MIME preview of ") (buffer-name))))
+    (erase-buffer)
+    (insert-buffer buf)
+    (if (re-search-forward
+	 (concat "^" (regexp-quote mail-header-separator) "\n") nil t)
+	(replace-match "\n"))
+    (mml-to-mime)
+    (unless raw
+      (run-hooks 'gnus-article-decode-hook)
+      (let ((gnus-newsgroup-name "dummy"))
+	(gnus-article-prepare-display)))
+    (fundamental-mode)
+    (setq buffer-read-only t)
+    (goto-char (point-min))))
 
 (defun mml-validate ()
   "Validate the current MML document."
