@@ -377,11 +377,12 @@
 	      (narrow-to-region (point) (search-forward "</story>"))
 	      (goto-char (point-min))
 	      (re-search-forward "<title>\\([^<]+\\)</title>")
-	      (setq description (nnweb-decode-entities-string (match-string 1)))
+	      (setq description
+		    (nnweb-decode-entities-string (match-string 1)))
 	      (re-search-forward "<url>\\([^<]+\\)</url>")
 	      (setq sid (match-string 1))
-	      (string-match "/\\([0-9/]+\\).shtml" sid)
-	      (setq sid (match-string 1 sid))
+	      (string-match "/\\([0-9/]+\\)\\(.shtml\\|$\\)" sid)
+	      (setq sid (concat "00/" (match-string 1 sid)))
 	      (re-search-forward "<comments>\\([^<]+\\)</comments>")
 	      (setq articles (string-to-number (match-string 1)))
 	      (setq gname (concat description " (" sid ")"))
@@ -397,9 +398,11 @@
 		(nnweb-insert (format nnslashdot-active-url number) t)
 		(goto-char (point-min))
 		(while (re-search-forward
-			"article.pl\\?sid=\\([^&]+\\).*<b>\\([^<]+\\)</b>" nil t)
+			"article.pl\\?sid=\\([^&]+\\).*<b>\\([^<]+\\)</b>"
+			nil t)
 		  (setq sid (match-string 1)
-			description (nnweb-decode-entities-string (match-string 2)))
+			description
+			(nnweb-decode-entities-string (match-string 2)))
 		  (forward-line 1)
 		  (when (re-search-forward "<b>\\([0-9]+\\)</b>" nil t)
 		    (setq articles (string-to-number (match-string 1))))
